// Define the charting module, specifying its dependencies.
// This module gets passed data and renders it as a chart using C3.
define([    'jquery', 'c3'],
	function($      ,  c3){
		var js_gh_messages;
		var label_formatters;
		var data_awaiting_formatter;

		// Create the public object for this module
		var charting = {};
	
		charting.display_chart = function(metrics){
			if(!label_formatters) {
				data_awaiting_formatter = metrics;
				return;
			}
			
			var columns = [];
			var y_to_x_column_mapping = {};
			var column_labels = {};
			
			
			// Could avoid this loop by using "rows:" rather than "columns:", but then would have to load one line at once - probably worth it.
			// or using "json:" and "x:"
			$.each(metrics, function(index, metric) {
				if(metric) {
					var data = metric.results[0].values;
					var x = [];
					var y = [];
					$.each(data, function(index, point){
						x[index] = point[0];
						y[index] = point[1];
					});
					columns.push(x);
					columns.push(y);
					
					var y_data_column_name = metric.results[0].name;
					y.unshift(y_data_column_name);
					
					var x_data_column_name = y_data_column_name + '.x'; 
					x.unshift(x_data_column_name);
					
					var column_label;
					if(y_data_column_name.indexOf("evt_in") !== -1) {
						column_label = js_gh_messages.metricsView_callsReceived();
					} else if (y_data_column_name.indexOf("evt_match") !== -1) {
						column_label = js_gh_messages.metricsView_callsVirtualized();
					} else {
						column_label = y_data_column_name;
					}
					column_labels[y_data_column_name] = column_label;
					
					y_to_x_column_mapping[y_data_column_name] = x_data_column_name;
				}
			});
	
			var chart_settings = {
				bindto : '#chart',
				data : {
					xs : y_to_x_column_mapping,
					columns : columns,
					type : 'bar',
					names: column_labels
				},
				axis : {
					x : {
						type : 'timeseries',
						tick : {
							format : label_formatters.x_axis_formatter,
							fit : true,
							rotate: 45
						},
						height: 130
					},
					y: {
						tick: {
							format: label_formatters.y_axis_formatter
						}
					}
				},
				legend: {
					position: "right"
				},
				bar : {
					width : {
						ratio : 1
					}
				}
			};
	
			$("#error").hide();
			$(".part-of-chart").fadeTo(0, 1);
			
			var chart = $("#chart");
			chart.html('');
			chart.show();
			
			$("#loading").hide();
			
			c3.generate(chart_settings);
		};
		
		charting.set_is_loading = function(loading_message) {
			$(".part-of-chart").fadeTo('short', 0.4);
			
			var loading = $("#loading");
			loading.fadeTo(0, 0);
			loading.text(loading_message);
			loading.show();
			loading.fadeTo('short', 1);
		};
		
		// For other parts of the code to call if there is an error loading a chart.
		// (Other errors should be reported by other means, as this hides the chart).
		charting.display_loading_error = function(message) {
			$("#chart").hide();
			$("#loading").hide();
			
			var element = $("#error");
			element.text(message);
			element.show();
		};
		
		charting.set_messages = function(messages) {
			// We ought to move the usage of translated message into MetricsChartPresenter
			// or similar and remove this function
			js_gh_messages = messages;
		}
		
		charting.set_formatters = function(formatters) {
			label_formatters = formatters;
			if(data_awaiting_formatter) {
				charting.display_chart(data_awaiting_formatter);
				data_awaiting_formatter = null;
			}
		}

		// Return the object that will be passed to other modules if they state
		// a dependency on 'rtcp/charting' (through calls to require.js)
		return charting;
	}
);